#include <math.h>
#include "arbre.h"

Arbre::Arbre() : coltronc(100,100,50), colfeuil(0,200,0)
{
  incl     = 0;
  maxprof  = 5;
  epais    = false;
  feuilles = false;
  altern   = false;
}

Arbre::~Arbre()
{
}

void Arbre::ajouteBranche(int l, int a)
{
  Branche br;
  br.taille = l;
  br.angle    = a;
  branches.append(br);
}

void Arbre::setBranche(int n,int l, int a)
{
  branches[n].taille = l;
  branches[n].angle    = a;
}

void Arbre::Dessine(QPainter *painter, bool model, qreal t, int p, int mp)
{
  maxprof = mp;
  QApplication::setOverrideCursor(Qt::WaitCursor);
  Tortue *tort = new Tortue(painter);
  if (model) tort->setOrigin(painter->window().width()/2,painter->window().height()-10);
  else       tort->setOrigin(painter->window().width()/2,painter->window().height()-50);
  tort->reset();
  if (!model) tort->setColor(coltronc);
  Arborise(tort, model, t, p);
  if (model) poignees = tort->Poignees();
  delete tort;
//   qDebug() << QString("Embranchement en %1 x %2").arg(embranch.x()).arg(embranch.y());
  QApplication::restoreOverrideCursor();
}

void Arbre::Arborise(Tortue *tort, bool model, qreal t, int p)
{
  if (p==0) {
    if (!model && feuilles) tort->ploot(colfeuil);
    return;
  }
  bool inverser = (altern) ? (((maxprof-p) % 2) != 0) : false;
  tort->push();
  if (!model && epais) tort->setWidth(2*p/3);
  tort->avance(t);
  tort->tourneDroite(incl);
//  tort->tourneDroite((inverser) ? -incl : incl);
  if (model && (p==2)) {
    QColor axe = Qt::red;
    tort->push();
    tort->setColor(axe);
    tort->setStyle(Qt::DashLine);
    tort->avance(100);
    tort->plot(axe);
    tort->setColor(Qt::black);
    tort->setStyle(Qt::SolidLine);
    tort->pop();
  }
  if (model && (p==2)) embranch = tort->Pos().toPoint();
  if (model && (p==1)) tort->plot();
  for(int i=0;i<count();++i)
  {
    tort->push();
//    tort->tourneDroite(branches.at(i).angle);
    tort->tourneDroite((inverser) ? -branches.at(i).angle : branches.at(i).angle);
    Arborise(tort, model, t*branches.at(i).taille/100.0,p-1);
    tort->pop();
  }
  tort->pop();
}

int Arbre::getPoignee(int x, int y)
{
  for(int i=0;i<poignees.size();++i) {
    if (poignees.at(i).contains(x,y)) return i;
  }
  return -1;
}
void Arbre::dragPoignee(int n, int x, int y, qreal t)
{
  int dx = x-embranch.x();
  int dy = embranch.y()-y;
  int angle  = qRound(180.0*atan2(dx,dy)/Pi);
  if (n>0) angle -= incl;
  int taille = qRound(100.0*sqrt((dx*dx)+(dy*dy))/t);
  if (angle < -120) angle = -120;
  if (angle >  120) angle =  120;
  if (taille > 90) taille = 90;
  if (n==0) incl = angle;
  if (n>0) {
    branches[n-1].angle  = angle;
    branches[n-1].taille = taille;
  }
//   qDebug() << QString("%1 : %2 x %3 : %4").arg(n).arg(dx).arg(dy).arg(angle);
}
