#include "grafarbre.h"

//---------------------------
VueArbre::VueArbre(QWidget *parent)
  : QFrame(parent)
{
  setBackgroundRole(QPalette::Base);
  longueur = 100;
  prof     = 5;
}

void VueArbre::paintEvent(QPaintEvent *e)
{
  QFrame::paintEvent(e);
  QPainter *painter = new QPainter(this);
  arbre->Dessine(painter, false, longueur, prof, arbre->maxProf());
  delete painter;
}
//---------------------------
VueModele::VueModele(QWidget *parent)
  : QFrame(parent)
{
  dragging = -1;
  setBackgroundRole(QPalette::Base);
}

void VueModele::paintEvent(QPaintEvent *e)
{
  QFrame::paintEvent(e);
  QPainter *painter = new QPainter(this);
  arbre->Dessine(painter, true, 80, 2, 2);
  delete painter;
}

void VueModele::mousePressEvent(QMouseEvent *e)
{
  int n = arbre->getPoignee(e->x(), e->y());
  if (n==-1) return;
  dragging = n;
  if (n>0){
    ((Grafarbre *) parent())->setBranche(n);
  }
}

void VueModele::mouseMoveEvent(QMouseEvent *e)
{
  arbre->dragPoignee(dragging, e->x(), e->y(), 80);
  if (dragging==0) ((Grafarbre *) parent())->retourIncline();
  if (dragging>0)  ((Grafarbre *) parent())->lireBranche(dragging);
  ((Grafarbre *) parent())->rafraichi();
}

void VueModele::mouseReleaseEvent(QMouseEvent *)
{
  dragging = -1;
  ((Grafarbre *) parent())->relache();
}

//---------------------------
AboutDialog::AboutDialog(QWidget *parent) : QDialog(parent)
{
  ui.setupUi(this);
  connect(ui.btFermer,  SIGNAL(clicked()), this, SLOT(close()));
  connect(ui.btAboutQt, SIGNAL(clicked()), qApp, SLOT(aboutQt()));
}
//---------------------------
Grafarbre::Grafarbre()
{
  ui.setupUi(this);
//   setWindowFlags(Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint);
  arbre = new Arbre();
  arbre->ajouteBranche(80,0);
  arbre->ajouteBranche(50,-30);
  arbre->ajouteBranche(50,30);
  ui.semis->setArbre(arbre);
  ui.jardin->setArbre(arbre);
  ui.spLongueur->setValue(ui.jardin->getLongueur());
  ui.spBranche->setValue(1);
  ui.spTaille->setValue(arbre->lireTaille(0));
  ui.spAngle->setValue(arbre->lireAngle(0));
  maj         = true;
  vent        = 1;
  ventmax     = 20;

  QObject::connect(ui.btFermer,      SIGNAL(clicked()),         this,      SLOT(close()));
  QObject::connect(ui.btAbout,       SIGNAL(clicked()),         this,      SLOT(about()));
  QObject::connect(ui.spLongueur,    SIGNAL(valueChanged(int)), ui.jardin, SLOT(setLongueur(int)));
  QObject::connect(ui.spProfondeur,  SIGNAL(valueChanged(int)), ui.jardin, SLOT(setProfondeur(int)));
  QObject::connect(ui.spBranche,     SIGNAL(valueChanged(int)), this,      SLOT(lireBranche(int)));
  QObject::connect(ui.spTaille,      SIGNAL(valueChanged(int)), this,      SLOT(setBranche()));
  QObject::connect(ui.spAngle,       SIGNAL(valueChanged(int)), this,      SLOT(setBranche()));
  QObject::connect(ui.ckVent,        SIGNAL(stateChanged(int)), this,      SLOT(setVent(int)));
  QObject::connect(ui.ckEpais,       SIGNAL(stateChanged(int)), this,      SLOT(epaissir(int)));
  QObject::connect(ui.ckAlterne,     SIGNAL(stateChanged(int)), this,      SLOT(alterner(int)));
  QObject::connect(ui.ckFeuilles,    SIGNAL(stateChanged(int)), this,      SLOT(feuillus(int)));
  QObject::connect(ui.spInclinaison, SIGNAL(valueChanged(int)), this,      SLOT(setIncline(int)));
  QObject::connect(ui.btPlus,        SIGNAL(clicked()),         this,      SLOT(plusBranche()));
  QObject::connect(ui.btMoins,       SIGNAL(clicked()),         this,      SLOT(moinsBranche()));
  QObject::connect(ui.btColTronc,    SIGNAL(clicked()),         this,      SLOT(lireColTronc()));
  QObject::connect(ui.btColFeuil,    SIGNAL(clicked()),         this,      SLOT(lireColFeuil()));

  ui.jardin->setProfondeur(ui.spProfondeur->value());
}

Grafarbre::~Grafarbre()
{
  delete arbre;
}

void Grafarbre::resizeEvent(QResizeEvent *)
{
  ui.jardin->resize(width()-ui.jardin->x()-10, height()-ui.jardin->y()-10);
}

void Grafarbre::timerEvent(QTimerEvent *)
{
  int incl = ui.spInclinaison->value();
  if (incl >  abs(ventmax)) vent = -1;
  if (incl < -abs(ventmax)) vent =  1;
  incl += vent;
  ui.spInclinaison->setValue(incl);
}

void Grafarbre::about()
{
  AboutDialog ab(this);
  ab.setVersion(APP_VERSION);
  ab.show();
  ab.exec();
}

void Grafarbre::lireBranche(int i)
{
  maj = false;
  ui.spTaille->setValue(arbre->lireTaille(i-1));
  ui.spAngle->setValue(arbre->lireAngle(i-1));
  maj = true;
}

void Grafarbre::setBranche()
{
  if (maj) {
    arbre->setBranche(ui.spBranche->value()-1, ui.spTaille->value(), ui.spAngle->value());
    ui.semis->update();
    ui.jardin->update();
  }
}

void Grafarbre::setIncline(int i)
{
  arbre->setIncline(i);
  ui.semis->update();
  ui.jardin->update();
}

void Grafarbre::setVent(int i)
{
//   QMessageBox::information(this,"Info",QString("%1").arg(i));
  if (i) timerId = startTimer(50);
  else   killTimer(timerId);
}

void Grafarbre::plusBranche()
{
  if (arbre->count()>5) return;
  maj = false;
  arbre->ajouteBranche(30, arbre->lireAngle(arbre->count()-1)+10);
  ui.spBranche->setMaximum(arbre->count());
  ui.spBranche->setValue(arbre->count());
  ui.btMoins->setEnabled(true);
  maj = true;
  ui.semis->update();
  ui.jardin->update();
}

void Grafarbre::moinsBranche()
{
  maj = false;
  arbre->effaceBranche(ui.spBranche->value()-1);
  ui.spBranche->setMaximum(arbre->count());
  if (arbre->count()<2) ui.btMoins->setEnabled(false);
  maj = true;
  lireBranche(ui.spBranche->value());
  ui.semis->update();
  ui.jardin->update();
}
void Grafarbre::relache()
{
  ui.jardin->update();
}

void Grafarbre::lireColTronc()
{
  QColor color = QColorDialog::getColor(arbre->getColTronc(), this);
  if (color.isValid()) arbre->setColTronc(color);
  ui.jardin->update();
}

void Grafarbre::lireColFeuil()
{
  QColor color = QColorDialog::getColor(arbre->getColFeuil(), this);
  if (color.isValid()) arbre->setColFeuil(color);
  ui.jardin->update();
}
